@@ -18,6 +18,7 @@
 #include <math.h>
 #include <stdatomic.h>
 #include <stdbool.h>
+#include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -30,8 +31,7 @@
 #include <ortho/ortho.h>
 #endif
 
-#define NSUB 9    /* number of subdivisions, re-aiming splines */
-#define CHUNK 128 /* in building list of edges */
+#define NSUB 9 /* number of subdivisions, re-aiming splines */
 
 #define MINW 16 /* minimum width of a box in the edge path */
 #define HALFMINW 8
@@ -43,22 +43,21 @@
 #define AUXGRAPH 128
 #define GRAPHTYPEMASK 192 /* the OR of the above */
 
-#define MAKEFWDEDGE(new, old)                                                  \
-  {                                                                            \
-    edge_t *newp;                                                              \
-    Agedgeinfo_t *info;                                                        \
-    newp = new;                                                                \
-    info = (Agedgeinfo_t *)newp->base.data;                                    \
-    *info = *(Agedgeinfo_t *)old->base.data;                                   \
-    *newp = *old;                                                              \
-    newp->base.data = (Agrec_t *)info;                                         \
-    AGTAIL(newp) = AGHEAD(old);                                                \
-    AGHEAD(newp) = AGTAIL(old);                                                \
-    ED_tail_port(newp) = ED_head_port(old);                                    \
-    ED_head_port(newp) = ED_tail_port(old);                                    \
-    ED_edge_type(newp) = VIRTUAL;                                              \
-    ED_to_orig(newp) = old;                                                    \
-  }
+static void makefwdedge(edge_t *new, edge_t *old) {
+  Agedgeinfo_t *const info =
+      (Agedgeinfo_t *)((uintptr_t)new->base.data - offsetof(Agedgeinfo_t, hdr));
+  const Agedgeinfo_t *const old_info =
+      (Agedgeinfo_t *)((uintptr_t)old->base.data - offsetof(Agedgeinfo_t, hdr));
+  *info = *old_info;
+  *new = *old;
+  new->base.data = &info->hdr;
+  AGTAIL(new) = AGHEAD(old);
+  AGHEAD(new) = AGTAIL(old);
+  ED_tail_port(new) = ED_head_port(old);
+  ED_head_port(new) = ED_tail_port(old);
+  ED_edge_type(new) = VIRTUAL;
+  ED_to_orig(new) = old;
+}
 
 typedef struct {
   double LeftBound;
@@ -68,7 +67,7 @@ typedef struct {
   boxf *Rank_box;
 } spline_info_t;
 
-DEFINE_LIST(points, pointf)
+typedef LIST(pointf) points_t;
 
 static void adjustregularpath(path *, size_t, size_t);
 static Agedge_t *bot_bound(Agedge_t *, int);
@@ -95,11 +94,6 @@ static int straight_len(Agnode_t *);
 static Agedge_t *straight_path(Agedge_t *, int, points_t *);
 static Agedge_t *top_bound(Agedge_t *, int);
 
-#define GROWEDGES                                                              \
-  do {                                                                         \
-    edges = gv_recalloc(edges, n_edges, n_edges + CHUNK, sizeof(edge_t *));    \
-  } while (0)
-
 static edge_t *getmainedge(edge_t *e) {
   edge_t *le = e;
   while (ED_to_virt(le))
@@ -163,7 +157,7 @@ static void swap_spline(splines *s) {
     SWAP(&s->list[i], &s->list[sz - 1 - i]);
   }
 
-  // swap beziers
+  // swap Béziers
   for (size_t i = 0; i < sz; ++i) {
     swap_bezier(&s->list[i]);
   }
@@ -240,11 +234,11 @@ static int dot_splines_(graph_t *g, int normalize) {
   node_t *n;
   Agedgeinfo_t fwdedgeai, fwdedgebi;
   Agedgepair_t fwdedgea, fwdedgeb;
-  edge_t *e, *e0, *e1, *ea, *eb, *le0, *le1, **edges = NULL;
+  edge_t *e, *e0, *e1, *ea, *eb, *le0, *le1;
   path P = {0};
   int et = EDGE_TYPE(g);
-  fwdedgea.out.base.data = (Agrec_t *)&fwdedgeai;
-  fwdedgeb.out.base.data = (Agrec_t *)&fwdedgebi;
+  fwdedgea.out.base.data = &fwdedgeai.hdr;
+  fwdedgeb.out.base.data = &fwdedgebi.hdr;
 
   if (et == EDGETYPE_NONE)
     return 0;
@@ -256,6 +250,7 @@ static int dot_splines_(graph_t *g, int normalize) {
     }
   }
   spline_info_t sd = {0};
+  LIST(edge_t *) edges = {0};
 #ifdef ORTHO
   if (et == EDGETYPE_ORTHO) {
     resetRW(g);
@@ -275,10 +270,8 @@ static int dot_splines_(graph_t *g, int normalize) {
     return 0;
   sd = (spline_info_t){.Splinesep = GD_nodesep(g) / 4,
                        .Multisep = GD_nodesep(g)};
-  edges = gv_calloc(CHUNK, sizeof(edge_t *));
 
   /* compute boundaries and list of splines */
-  unsigned n_edges = 0;
   n_nodes = 0;
   for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
     n_nodes += GD_rank(g)[i].n;
@@ -306,16 +299,12 @@ static int dot_splines_(graph_t *g, int normalize) {
         if (ED_edge_type(e) == FLATORDER || ED_edge_type(e) == IGNORED)
           continue;
         setflags(e, REGULAREDGE, FWDEDGE, MAINGRAPH);
-        edges[n_edges++] = e;
-        if (n_edges % CHUNK == 0)
-          GROWEDGES;
+        LIST_APPEND(&edges, e);
       }
       if (ND_flat_out(n).list)
         for (int k = 0; (e = ND_flat_out(n).list[k]); k++) {
           setflags(e, FLATEDGE, 0, AUXGRAPH);
-          edges[n_edges++] = e;
-          if (n_edges % CHUNK == 0)
-            GROWEDGES;
+          LIST_APPEND(&edges, e);
         }
       if (ND_other(n).list) {
         /* In position, each node has its rw stored in mval and,
@@ -328,9 +317,7 @@ static int dot_splines_(graph_t *g, int normalize) {
         }
         for (int k = 0; (e = ND_other(n).list[k]); k++) {
           setflags(e, 0, 0, AUXGRAPH);
-          edges[n_edges++] = e;
-          if (n_edges % CHUNK == 0)
-            GROWEDGES;
+          LIST_APPEND(&edges, e);
         }
       }
     }
@@ -342,7 +329,7 @@ static int dot_splines_(graph_t *g, int normalize) {
    * alternatively, the edges would be routed identically if
    * routed separately.
    */
-  qsort(edges, n_edges, sizeof(edges[0]), edgecmp);
+  LIST_SORT(&edges, edgecmp);
 
   /* FIXME: just how many boxes can there be? */
   P.boxes = gv_calloc(n_nodes + 20 * 2 * NSUB, sizeof(boxf));
@@ -357,21 +344,21 @@ static int dot_splines_(graph_t *g, int normalize) {
     }
   }
 
-  for (unsigned l = 0; l < n_edges;) {
+  for (unsigned l = 0; l < LIST_SIZE(&edges);) {
     const unsigned ind = l;
-    le0 = getmainedge((e0 = edges[l++]));
+    le0 = getmainedge((e0 = LIST_GET(&edges, l++)));
     if (ED_tail_port(e0).defined || ED_head_port(e0).defined) {
       ea = e0;
     } else {
       ea = le0;
     }
     if (ED_tree_index(ea) & BWDEDGE) {
-      MAKEFWDEDGE(&fwdedgea.out, ea);
+      makefwdedge(&fwdedgea.out, ea);
       ea = &fwdedgea.out;
     }
     unsigned cnt;
-    for (cnt = 1; l < n_edges; cnt++, l++) {
-      if (le0 != (le1 = getmainedge((e1 = edges[l]))))
+    for (cnt = 1; l < LIST_SIZE(&edges); cnt++, l++) {
+      if (le0 != (le1 = getmainedge((e1 = LIST_GET(&edges, l)))))
         break;
       if (ED_adjacent(e0))
         continue; /* all flat adjacent edges at once */
@@ -381,7 +368,7 @@ static int dot_splines_(graph_t *g, int normalize) {
         eb = le1;
       }
       if (ED_tree_index(eb) & BWDEDGE) {
-        MAKEFWDEDGE(&fwdedgeb.out, eb);
+        makefwdedge(&fwdedgeb.out, eb);
         eb = &fwdedgeb.out;
       }
       if (portcmp(ED_tail_port(ea), ED_tail_port(eb)))
@@ -391,15 +378,15 @@ static int dot_splines_(graph_t *g, int normalize) {
       if ((ED_tree_index(e0) & EDGETYPEMASK) == FLATEDGE &&
           ED_label(e0) != ED_label(e1))
         break;
-      if (ED_tree_index(edges[l]) & MAINGRAPH) /* Aha! -C is on */
+      if (ED_tree_index(LIST_GET(&edges, l)) & MAINGRAPH) /* Aha! -C is on */
         break;
     }
 
     if (et == EDGETYPE_CURVED) {
       edge_t **edgelist = gv_calloc(cnt, sizeof(edge_t *));
-      edgelist[0] = getmainedge((edges + ind)[0]);
+      edgelist[0] = getmainedge(LIST_GET(&edges, ind));
       for (unsigned ii = 1; ii < cnt; ii++)
-        edgelist[ii] = (edges + ind)[ii];
+        edgelist[ii] = LIST_GET(&edges, ind + ii);
       makeStraightEdges(g, edgelist, cnt, et, &sinfo);
       free(edgelist);
     } else if (agtail(e0) == aghead(e0)) {
@@ -418,20 +405,24 @@ static int dot_splines_(graph_t *g, int normalize) {
         double dwny = ND_coord(n).y - ND_coord(GD_rank(g)[r + 1].v[0]).y;
         sizey = MIN(upy, dwny);
       }
-      makeSelfEdge(edges, ind, cnt, sd.Multisep, sizey / 2, &sinfo);
+      makeSelfEdge(LIST_FRONT(&edges), ind, cnt, sd.Multisep, sizey / 2,
+                   &sinfo);
       for (unsigned b = 0; b < cnt; b++) {
-        e = edges[ind + b];
+        e = LIST_GET(&edges, ind + b);
         if (ED_label(e))
           updateBB(g, ED_label(e));
       }
     } else if (ND_rank(agtail(e0)) == ND_rank(aghead(e0))) {
-      const int rc = make_flat_edge(g, sd, &P, edges, ind, cnt, et);
+      const int rc =
+          make_flat_edge(g, sd, &P, LIST_FRONT(&edges), ind, cnt, et);
       if (rc != 0) {
         free(sd.Rank_box);
+        LIST_FREE(&edges);
+        free(P.boxes);
         return rc;
       }
     } else
-      make_regular_edge(g, &sd, &P, edges, ind, cnt, et);
+      make_regular_edge(g, &sd, &P, LIST_FRONT(&edges), ind, cnt, et);
   }
 
   /* place regular edge labels */
@@ -480,7 +471,7 @@ finish:
     routesplinesterm();
   }
   free(sd.Rank_box);
-  free(edges);
+  LIST_FREE(&edges);
   free(P.boxes);
   State = GVSPLINES;
   EdgeLabelsDone = 1;
@@ -549,28 +540,18 @@ static void setflags(edge_t *e, int hint1, int hint2, int f3) {
  *  - labels if flat edges
  *  - edge id
  */
-static int edgecmp(const void *x, const void *y) {
-// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
-// as the later usage is const. We need the cast because the macros use
-// non-const pointers for genericity.
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-  edge_t **ptr0 = (edge_t **)x;
-  edge_t **ptr1 = (edge_t **)y;
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
+static int edgecmp(const void *p0, const void *p1) {
+  edge_t *const *ptr0 = p0;
+  edge_t *const *ptr1 = p1;
   Agedgeinfo_t fwdedgeai, fwdedgebi;
   Agedgepair_t fwdedgea, fwdedgeb;
-  edge_t *e0, *e1, *ea, *eb, *le0, *le1;
+  edge_t *ea, *eb, *le0, *le1;
   int et0, et1, rv;
 
-  fwdedgea.out.base.data = (Agrec_t *)&fwdedgeai;
-  fwdedgeb.out.base.data = (Agrec_t *)&fwdedgebi;
-  e0 = *ptr0;
-  e1 = *ptr1;
+  fwdedgea.out.base.data = &fwdedgeai.hdr;
+  fwdedgeb.out.base.data = &fwdedgebi.hdr;
+  edge_t *const e0 = *ptr0;
+  edge_t *const e1 = *ptr1;
   et0 = ED_tree_index(e0) & EDGETYPEMASK;
   et1 = ED_tree_index(e1) & EDGETYPEMASK;
   if (et0 < et1) {
@@ -619,12 +600,12 @@ static int edgecmp(const void *x, const void *y) {
 
   ea = (ED_tail_port(e0).defined || ED_head_port(e0).defined) ? e0 : le0;
   if (ED_tree_index(ea) & BWDEDGE) {
-    MAKEFWDEDGE(&fwdedgea.out, ea);
+    makefwdedge(&fwdedgea.out, ea);
     ea = &fwdedgea.out;
   }
   eb = (ED_tail_port(e1).defined || ED_head_port(e1).defined) ? e1 : le1;
   if (ED_tree_index(eb) & BWDEDGE) {
-    MAKEFWDEDGE(&fwdedgeb.out, eb);
+    makefwdedge(&fwdedgeb.out, eb);
     eb = &fwdedgeb.out;
   }
   if ((rv = portcmp(ED_tail_port(ea), ED_tail_port(eb))))
@@ -1568,13 +1549,13 @@ static int make_flat_edge(graph_t *g, const spline_info_t sp, path *P,
   int tside, hside;
   pathend_t tend, hend;
 
-  fwdedge.out.base.data = (Agrec_t *)&fwdedgei;
+  fwdedge.out.base.data = &fwdedgei.hdr;
 
   /* Get sample edge; normalize to go from left to right */
   e = edges[ind];
   bool isAdjacent = ED_adjacent(e) != 0;
   if (ED_tree_index(e) & BWDEDGE) {
-    MAKEFWDEDGE(&fwdedge.out, e);
+    makefwdedge(&fwdedge.out, e);
     e = &fwdedge.out;
   }
   for (unsigned i = 1; i < cnt; i++) {
@@ -1740,19 +1721,19 @@ static int makeLineEdge(graph_t *g, edge_t *fe, points_t *points, node_t **hp) {
       lp.y += height / 2.0;
     }
 
-    points_append(points, startp);
-    points_append(points, startp);
-    points_append(points, lp);
-    points_append(points, lp);
-    points_append(points, lp);
-    points_append(points, endp);
-    points_append(points, endp);
+    LIST_APPEND(points, startp);
+    LIST_APPEND(points, startp);
+    LIST_APPEND(points, lp);
+    LIST_APPEND(points, lp);
+    LIST_APPEND(points, lp);
+    LIST_APPEND(points, endp);
+    LIST_APPEND(points, endp);
     pn = 7;
   } else {
-    points_append(points, startp);
-    points_append(points, startp);
-    points_append(points, endp);
-    points_append(points, endp);
+    LIST_APPEND(points, startp);
+    LIST_APPEND(points, startp);
+    LIST_APPEND(points, endp);
+    LIST_APPEND(points, endp);
     pn = 4;
   }
 
@@ -1772,26 +1753,28 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
   points_t pointfs = {0};
   points_t pointfs2 = {0};
 
-  fwdedgea.out.base.data = (Agrec_t *)&fwdedgeai;
-  fwdedgeb.out.base.data = (Agrec_t *)&fwdedgebi;
-  fwdedge.out.base.data = (Agrec_t *)&fwdedgei;
+  fwdedgea.out.base.data = &fwdedgeai.hdr;
+  fwdedgeb.out.base.data = &fwdedgebi.hdr;
+  fwdedge.out.base.data = &fwdedgei.hdr;
 
   sl = 0;
   e = edges[ind];
   bool hackflag = false;
   if (abs(ND_rank(agtail(e)) - ND_rank(aghead(e))) > 1) {
-    fwdedgeai = *(Agedgeinfo_t *)e->base.data;
+    fwdedgeai = *(Agedgeinfo_t *)((uintptr_t)e->base.data -
+                                  offsetof(Agedgeinfo_t, hdr));
     fwdedgea.out = *e;
     fwdedgea.in = *AGOUT2IN(e);
-    fwdedgea.out.base.data = (Agrec_t *)&fwdedgeai;
+    fwdedgea.out.base.data = &fwdedgeai.hdr;
     if (ED_tree_index(e) & BWDEDGE) {
-      MAKEFWDEDGE(&fwdedgeb.out, e);
+      makefwdedge(&fwdedgeb.out, e);
       agtail(&fwdedgea.out) = aghead(e);
       ED_tail_port(&fwdedgea.out) = ED_head_port(e);
     } else {
-      fwdedgebi = *(Agedgeinfo_t *)e->base.data;
+      fwdedgebi = *(Agedgeinfo_t *)((uintptr_t)e->base.data -
+                                    offsetof(Agedgeinfo_t, hdr));
       fwdedgeb.out = *e;
-      fwdedgeb.out.base.data = (Agrec_t *)&fwdedgebi;
+      fwdedgeb.out.base.data = &fwdedgebi.hdr;
       agtail(&fwdedgea.out) = agtail(e);
       fwdedgeb.in = *AGOUT2IN(e);
     }
@@ -1807,7 +1790,7 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
     hackflag = true;
   } else {
     if (ED_tree_index(e) & BWDEDGE) {
-      MAKEFWDEDGE(&fwdedgea.out, e);
+      makefwdedge(&fwdedgea.out, e);
       e = &fwdedgea.out;
     }
   }
@@ -1832,7 +1815,7 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
     bool smode = false;
     si = -1;
     while (ND_node_type(hn) == VIRTUAL && !sinfo.splineMerge(hn)) {
-      boxes_append(&boxes, rank_box(sp, g, ND_rank(tn)));
+      LIST_APPEND(&boxes, rank_box(sp, g, ND_rank(tn)));
       if (!smode && ((sl = straight_len(hn)) >=
                      ((GD_has_labels(g->root) & EDGE_LABEL) ? 4 + 1 : 2 + 1))) {
         smode = true;
@@ -1840,7 +1823,7 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
       }
       if (!smode || si > 0) {
         si--;
-        boxes_append(&boxes, maximal_bbox(g, *sp, hn, e, ND_out(hn).list[0]));
+        LIST_APPEND(&boxes, maximal_bbox(g, *sp, hn, e, ND_out(hn).list[0]));
         e = ND_out(hn).list[0];
         tn = agtail(e);
         hn = aghead(e);
@@ -1868,14 +1851,14 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
       }
       if (pn == 0) {
         free(ps);
-        boxes_free(&boxes);
-        points_free(&pointfs);
-        points_free(&pointfs2);
+        LIST_FREE(&boxes);
+        LIST_FREE(&pointfs);
+        LIST_FREE(&pointfs2);
         return;
       }
 
       for (size_t i = 0; i < pn; i++) {
-        points_append(&pointfs, ps[i]);
+        LIST_APPEND(&pointfs, ps[i]);
       }
       free(ps);
       e = straight_path(ND_out(hn).list[0], sl, &pointfs);
@@ -1883,7 +1866,7 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
       segfirst = e;
       tn = agtail(e);
       hn = aghead(e);
-      boxes_clear(&boxes);
+      LIST_CLEAR(&boxes);
       tend.nb = maximal_bbox(g, *sp, tn, ND_in(tn).list[0], e);
       beginpath(P, e, REGULAREDGE, &tend, spline_merge(tn));
       b = makeregularend(tend.boxes[tend.boxn - 1], BOTTOM,
@@ -1893,7 +1876,7 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
       P->start.theta = -M_PI / 2, P->start.constrained = true;
       smode = false;
     }
-    boxes_append(&boxes, rank_box(sp, g, ND_rank(tn)));
+    LIST_APPEND(&boxes, rank_box(sp, g, ND_rank(tn)));
     b = hend.nb = maximal_bbox(g, *sp, hn, e, NULL);
     endpath(P, hackflag ? &fwdedgeb.out : e, REGULAREDGE, &hend,
             spline_merge(aghead(e)));
@@ -1903,7 +1886,7 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
     if (b.LL.x < b.UR.x && b.LL.y < b.UR.y)
       hend.boxes[hend.boxn++] = b;
     completeregularpath(P, segfirst, e, &tend, &hend, &boxes);
-    boxes_free(&boxes);
+    LIST_FREE(&boxes);
     pointf *ps = NULL;
     size_t pn = 0;
     if (is_spline)
@@ -1921,12 +1904,12 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
     }
     if (pn == 0) {
       free(ps);
-      points_free(&pointfs);
-      points_free(&pointfs2);
+      LIST_FREE(&pointfs);
+      LIST_FREE(&pointfs2);
       return;
     }
     for (size_t i = 0; i < pn; i++) {
-      points_append(&pointfs, ps[i]);
+      LIST_APPEND(&pointfs, ps[i]);
     }
     free(ps);
     recover_slack(segfirst, P);
@@ -1936,39 +1919,37 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
   /* make copies of the spline points, one per multi-edge */
 
   if (cnt == 1) {
-    points_sync(&pointfs);
-    clip_and_install(fe, hn, points_front(&pointfs), points_size(&pointfs),
-                     &sinfo);
-    points_free(&pointfs);
-    points_free(&pointfs2);
+    LIST_SYNC(&pointfs);
+    clip_and_install(fe, hn, LIST_FRONT(&pointfs), LIST_SIZE(&pointfs), &sinfo);
+    LIST_FREE(&pointfs);
+    LIST_FREE(&pointfs2);
     return;
   }
   const double dx = sp->Multisep * (cnt - 1) / 2;
-  for (size_t k = 1; k + 1 < points_size(&pointfs); k++)
-    points_at(&pointfs, k)->x -= dx;
+  for (size_t k = 1; k + 1 < LIST_SIZE(&pointfs); k++)
+    LIST_AT(&pointfs, k)->x -= dx;
 
-  for (size_t k = 0; k < points_size(&pointfs); k++)
-    points_append(&pointfs2, points_get(&pointfs, k));
-  points_sync(&pointfs2);
-  clip_and_install(fe, hn, points_front(&pointfs2), points_size(&pointfs2),
-                   &sinfo);
+  for (size_t k = 0; k < LIST_SIZE(&pointfs); k++)
+    LIST_APPEND(&pointfs2, LIST_GET(&pointfs, k));
+  LIST_SYNC(&pointfs2);
+  clip_and_install(fe, hn, LIST_FRONT(&pointfs2), LIST_SIZE(&pointfs2), &sinfo);
   for (unsigned j = 1; j < cnt; j++) {
     e = edges[ind + j];
     if (ED_tree_index(e) & BWDEDGE) {
-      MAKEFWDEDGE(&fwdedge.out, e);
+      makefwdedge(&fwdedge.out, e);
       e = &fwdedge.out;
     }
-    for (size_t k = 1; k + 1 < points_size(&pointfs); k++)
-      points_at(&pointfs, k)->x += sp->Multisep;
-    points_clear(&pointfs2);
-    for (size_t k = 0; k < points_size(&pointfs); k++)
-      points_append(&pointfs2, points_get(&pointfs, k));
-    points_sync(&pointfs2);
-    clip_and_install(e, aghead(e), points_front(&pointfs2),
-                     points_size(&pointfs2), &sinfo);
+    for (size_t k = 1; k + 1 < LIST_SIZE(&pointfs); k++)
+      LIST_AT(&pointfs, k)->x += sp->Multisep;
+    LIST_CLEAR(&pointfs2);
+    for (size_t k = 0; k < LIST_SIZE(&pointfs); k++)
+      LIST_APPEND(&pointfs2, LIST_GET(&pointfs, k));
+    LIST_SYNC(&pointfs2);
+    clip_and_install(e, aghead(e), LIST_FRONT(&pointfs2), LIST_SIZE(&pointfs2),
+                     &sinfo);
   }
-  points_free(&pointfs);
-  points_free(&pointfs2);
+  LIST_FREE(&pointfs);
+  LIST_FREE(&pointfs2);
 }
 
 /* regular edges */
@@ -2001,9 +1982,9 @@ static void completeregularpath(path *P, edge_t *first, edge_t *last,
   for (int i = 0; i < tendp->boxn; i++)
     add_box(P, tendp->boxes[i]);
   const size_t fb = P->nbox + 1;
-  const size_t lb = fb + boxes_size(boxes) - 3;
-  for (size_t i = 0; i < boxes_size(boxes); i++)
-    add_box(P, boxes_get(boxes, i));
+  const size_t lb = fb + LIST_SIZE(boxes) - 3;
+  for (size_t i = 0; i < LIST_SIZE(boxes); i++)
+    add_box(P, LIST_GET(boxes, i));
   for (int i = hendp->boxn - 1; i >= 0; i--)
     add_box(P, hendp->boxes[i]);
   adjustregularpath(P, fb, lb);
@@ -2108,9 +2089,9 @@ static edge_t *straight_path(edge_t *e, int cnt, points_t *plist) {
 
   while (cnt--)
     f = ND_out(aghead(f)).list[0];
-  assert(!points_is_empty(plist));
-  points_append(plist, points_get(plist, points_size(plist) - 1));
-  points_append(plist, points_get(plist, points_size(plist) - 1));
+  assert(!LIST_IS_EMPTY(plist));
+  LIST_APPEND(plist, LIST_GET(plist, LIST_SIZE(plist) - 1));
+  LIST_APPEND(plist, LIST_GET(plist, LIST_SIZE(plist) - 1));
 
   return f;
 }
