@@ -134,7 +134,6 @@ int sfprint(FILE *f, Sffmt_t *format) {
 		    if (*t_str != '*')
 			n_str = (form - 1) - t_str;
 		    else {
-			t_str = _Sffmtintf(t_str + 1, &n);
 			++argn;
 
 			FMTSET(ft, form, LEFTP, 0, 0, 0, 0, 0, NULL, 0);
@@ -203,7 +202,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	  do_star:
 	    form += 1;		/* fall through */
 	case '*':
-	    form = _Sffmtintf(form, &n);
+	    form = _Sffmtintf(form);
 	    n = ++argn;
 
 	    FMTSET(ft, form, '.', dot, 0, 0, 0, 0, NULL, 0);
@@ -244,7 +243,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 		for (n = *form; gv_isdigit(n); n = *++form)
 		    size = size * 10 + (n - '0');
 	    } else if (*form == '*') {
-		form = _Sffmtintf(form + 1, &n);
+		form = _Sffmtintf(form + 1);
 		n = ++argn;
 
 		FMTSET(ft, form, 'I', sizeof(int), 0, 0, 0, 0, NULL, 0);
